/*****************************************************************************
* Copyright 2009 - 2009 Broadcom Corporation.  All rights reserved.
*
* Unless you and Broadcom execute a separate written software license
* agreement governing use of this software, this software is licensed to you
* under the terms of the GNU General Public License version 2, available at
* http://www.broadcom.com/licenses/GPLv2.php (the "GPL"). 
*
* Notwithstanding the above, under no circumstances may you combine this
* software in any way with any other Broadcom software provided under a
* license other than the GPL, without Broadcom's express prior written
* consent.
*****************************************************************************/

#ifndef BCM_ADC_H
#define BCM_ADC_H

#include <linux/list.h>

/*
 * Device and Channel information
 */
#if defined(CONFIG_PMU_DEVICE_BCM59040) || defined(CONFIG_BCM_PMU_BCM59040)
#define BCM_ADC_DEVICES				2
#else
#define BCM_ADC_DEVICES				1
#endif

#define BCM4760_ADC_DEVICE			0x0000
#define BCM4760_ADC_MIN_CHANNEL		0x0000
#define BCM4760_ADC_MAX_CHANNEL		0x0003

#if defined(CONFIG_PMU_DEVICE_BCM59040) || defined(CONFIG_BCM_PMU_BCM59040)
#define BCM59040_ADC_DEVICE			0x0001
#define BCM59040_ADC_MIN_CHANNEL	0x0000
#define BCM59040_ADC_MAX_CHANNEL	0x0009
#endif

/*
 * ADC Request structure used to send a request to the
 * bcm_adc_request routine to select the device/channel
 * of the request along with information to get the
 * response back to the caller.
 */
typedef struct bcm_adc_request_s
{
	struct list_head			list;
	int							device;
	int							channel;
	unsigned long int			param;
	void (*callback)(struct bcm_adc_request_s* req,
					 unsigned short sample,
					 int error);
} bcm_adc_request_t;

/*
 * completion handler definition
 */
typedef int (*bcm_adc_handler_t)(bcm_adc_request_t* req,
								 unsigned short sample,
								 int error);


/*
 * external interface prototypes
 */
extern int bcm4760_adc_request(bcm_adc_request_t*);
extern int bcm4760_adc_complete_register(bcm_adc_handler_t);

#if defined(CONFIG_PMU_DEVICE_BCM59040) || defined(CONFIG_BCM_PMU_BCM59040)
extern int bcm59040_adc_request(bcm_adc_request_t *);
extern int bcm59040_adc_complete_register(bcm_adc_handler_t);
#endif

/*
 * bcm_adc interface prototype
 */
extern int bcm_adc_request(bcm_adc_request_t* req);
extern int bcm_adc_cancel_request(bcm_adc_request_t* req);
#endif
